#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: satGeneral.cgi,v 1.29 2003/10/03 18:52:12 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/satGeneral.cgi,v $

########################################################################
# Perl script to format general status screen about satellites.
# We show lists of satellites being tracked, and a summary of the
# GPS constellation.  A timestamp is appended.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Satellites - General Information',
                         '/satGeneralInfo.html'
                       ) ;

open( SYSINFO, 
      '/usr/bin/sudo /usr/local/fireball/sysinfo -p tracking_status |');

# Response is twelve lines, one per channel of the form
#   0 :  15 51.7 93.3 D2FF 8001 50.8 41.8 35 2.8
#  Chn  PRN Elev Azm  Stat     
# Or (if channel is unused)
#   0 : idle

# Scan the list of Sv information, and extract a list
# of satellite's being tracked.  The definition of tracked is
# that the L1 status indicates measurements are being made <=> the
# 64-bit in the L1 status is set.
my @svList ;
my $i ;
for ( $i = 0 ; $i < 12 ; $i++ )
{
  if ( ($_ = <SYSINFO>) !~ /idle/ )
  {
    #print $_, "<BR>" ;
    ( undef, undef, my $prn, undef, undef, 
      my $l1stat, undef ) = split( ' ' ) ; 

    if ( hex($l1stat) & 64)
    {
      
      push @svList, $prn ;
    }
  }
  else
  {
    #print $_, "<BR>" ;
  }
}
close( SYSINFO ) ;


$_ = qx( /usr/local/fireball/sysinfo -p gps_hlth ) ;
# Sends back a string like:
# 0 0 0 0 63 0 0 0 0 0 0 -1 0 0 0 0 0 0 -1 0 0 -1 0 0 0 0 0 0 0 0 0 -1
# zero is healthy.  -1 is not-exist, otherwise value is an unhealthy 
# status code.
chomp ;
my @allHealth = split() ;

$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p sv_masks ) ;
# Return string should be of the form
#  put sv_masks E I E D E .... 32 of them.
my @state = split( " ", $_ ) ;
# Drop the "put sv_masks" portion.
splice( @state, 0, 2 ) ;




# Get Waas information...

$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_mode ) ;
# Response will be
#  put waas_mode on         (or off)

my $WaasEnabled = (split)[2] ;


$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_sv_masks ) ;
# Return string should be of the form
#   put waas_sv_masks D D D D D D D D D D D D D D E D D D D

my @waasState = split( " ", $_ ) ;
# Drop the "put sv_masks" portion.
splice( @waasState, 0, 2 ) ;




print "<B>Tracking ", scalar( @svList ), " Satellites</B>" ;
print '<Pre>' ;

# Sort the SV list numerically, replace the WAAS prns with their
# textual names, and output the list, separated by spaces.  
@svList = sort( {$a <=> $b} @svList ) ;
for my $prn (@svList)
{
      if    ($prn == 120) { $prn = 'EGNOS-AOR-E'; } 
      elsif ($prn == 122) { $prn = 'WAAS-AOR-W' ; }
      elsif ($prn == 129) { $prn = 'MSAS-1'        ; } 
      elsif ($prn == 131) { $prn = 'EGNOS-IOR'  ; }
      elsif ($prn == 134) { $prn = 'WAAS-POR'   ; }
      elsif ($prn == 137) { $prn = 'MSAS-2'        ; }
}
print join( ' ', @svList ) ;

print '</PRE>' ;


my @healthySVs ;
my @unhealthySVs ;
my @disabled ;
my @ignored ;

for ( my $i = 0 ; $i < 32 ; ++$i )
{
  if ($allHealth[$i] == 0)
  {
    push( @healthySVs, $i+1 ) ;
  }
  elsif ($allHealth[$i] != -1)
  {
    push( @unhealthySVs, $i+1 ) ;
  }

  if ($state[$i] eq 'D')
  {
    push( @disabled, $i+1 ) ;
  }
  elsif ($state[$i] eq 'I')
  {
    push( @ignored, $i+1 ) ;
  }
}


printf( '<B>Total GPS Constellation is %d satellites.</B>',
        scalar(@healthySVs) +scalar(@unhealthySVs) ) ; 


printf( '<P><B>%d Healthy Satellites :</B>', scalar(@healthySVs) );
print '<PRE>' ;
my $count = 0 ;
foreach my $prn (@healthySVs)
{
  printf( "%2d ", $prn ) ;
  if (++$count == 10)
  {
    print "\n" ;
    $count = 0 ;
  }
}
print '</PRE>' ;


printf( '<P><B>%d Unhealthy Satellites :</B>', scalar(@unhealthySVs) );
print '<PRE>' ;
$count = 0 ;
foreach my $prn (@unhealthySVs)
{
  printf( "%2d ", $prn ) ;
  if (++$count == 10)
  {
    print "\n" ;
    $count = 0 ;
  }
}
print '</PRE>' ;


print "<p><b>User Disabled Satellites:</b></p>" ;
print '<PRE>' ;
if (@disabled)
{
  foreach my $prn (@disabled)
  {
    printf( "%2d ", $prn ) ;
    if (++$count == 10)
    {
      print "\n" ;
      $count = 0 ;
    }
  }
}
else
{
  print "None" ;

}
print '</PRE>' ;


print "<p><b>User Ignore Health Satellites:</b></p>" ;
print '<PRE>' ;
if (@ignored)
{
  foreach my $prn (@ignored)
  {
    printf( "%2d ", $prn ) ;
    if (++$count == 10)
    {
      print "\n" ;
      $count = 0 ;
    }
  }
}
else
{
  print "None" ;

}
print '</PRE>' ;

if ( $WaasEnabled eq 'on' )
{
  print '<b>WAAS usage is enabled for</b><pre>' ;
  my $line = '' ;
  for ( my $prn = 120 ; $prn <= 138 ; ++$prn )
  {
    if ($waasState[$prn-120] =~ m/[EI]/)
    {
      my $name = "$prn" ; 
      if    ($prn == 120) { $name = 'EGNOS-AOR-E'; } 
      elsif ($prn == 122) { $name = 'WAAS-AOR-W' ; }
      elsif ($prn == 129) { $name = 'MSAS-1'     ; } 
      elsif ($prn == 131) { $name = 'EGNOS-IOR'  ; }
      elsif ($prn == 134) { $name = 'WAAS-POR'   ; }
      elsif ($prn == 137) { $name = 'MSAS-2'     ; }
      if (length($line) > 50 )
      {
        print $line, ",\n" ;
        $line = '' ;
      }
      $line .= ', ' if ($line) ;
      $line .= $name ;
    }
  }
  print $line, "\n" if length($line) ;
}
else
{
  print b('WAAS usage is disabled.') ;
}


print hr() ;

# Create Timestamp:
# 2001 Sep 6 - Thu of GPS Week 1130 - 17:46:52 GMT 
print timestamp() ;

# Refresh this page every five seconds.  
# We try to make this happen on exact 5 second boundaries.  If this time
# was an exact 5 second time, then we wait 4000 msec to allow a little
# bit of time for transmission and processing.  The range of delays is
# from 2000 to 6000.
my $delay = 6000 - ( (time()+2) % 5 ) * 1000 ;
print "
<script>
  parent.loadAfterDelay( window.location.href, $delay) ;
</script>
" ;

print end_html() ;

# end of script satGeneral.cgi
